@@ -1,10 +1,11 @@
 /*
 
-   BLIS    
+   BLIS
    An object-based framework for developing high-performance BLAS-like
    libraries.
 
    Copyright (C) 2014, The University of Texas at Austin
+   Copyright (C) 2018, Advanced Micro Devices, Inc.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are
@@ -14,9 +15,9 @@
     - Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
-    - Neither the name of The University of Texas at Austin nor the names
-      of its contributors may be used to endorse or promote products
-      derived from this software without specific prior written permission.
+    - Neither the name(s) of the copyright holder(s) nor the names of its
+      contributors may be used to endorse or promote products derived
+      from this software without specific prior written permission.
 
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
@@ -34,42 +35,42 @@
 
 #include "blis.h"
 
-unpackm_t* unpackm_cntl = NULL;
-
-void bli_unpackm_cntl_init()
+cntl_t* bli_unpackm_cntl_create_node
+     (
+       rntm_t*   rntm,
+       void*     var_func,
+       void*     unpackm_var_func,
+       cntl_t*   sub_node
+     )
 {
-	unpackm_cntl = bli_unpackm_cntl_obj_create( BLIS_UNBLOCKED,
-	                                            BLIS_VARIANT1,
-	                                            NULL ); // no blocksize needed
-}
+	cntl_t*           cntl;
+	unpackm_params_t* params;
 
-void bli_unpackm_cntl_finalize()
-{
-	bli_cntl_obj_free( unpackm_cntl );
-}
+	// NOTE: If this function is ever called, figure out whether the
+	// bli_malloc_intl() below needs to be changed to bli_sba_acquire().
+	bli_abort();
 
-unpackm_t* bli_unpackm_cntl_obj_create( impl_t     impl_type,
-                                        varnum_t   var_num,
-                                        blksz_t*   b )
-{
-	unpackm_t* cntl;
+	// Allocate an unpackm_params_t struct.
+	params = bli_malloc_intl( sizeof( unpackm_params_t ) );
 
-	cntl = ( unpackm_t* ) bli_malloc_intl( sizeof(unpackm_t) );
+	// Initialize the unpackm_params_t struct.
+	params->size      = sizeof( unpackm_params_t );
+	params->var_func  = unpackm_var_func;
 
-	cntl->impl_type = impl_type;
-	cntl->var_num   = var_num;
-	cntl->b         = b;
+	// It's important that we set the bszid field to BLIS_NO_PART to indicate
+	// that no blocksize partitioning is performed. bli_cntl_free() will rely
+	// on this information to know how to step through the thrinfo_t tree in
+	// sync with the cntl_t tree.
+	cntl = bli_cntl_create_node
+	(
+	  rntm,
+	  BLIS_NOID,
+	  BLIS_NO_PART,
+	  var_func,
+	  params,
+	  sub_node
+	);
 
 	return cntl;
 }
 
-void bli_unpackm_cntl_obj_init( unpackm_t* cntl,
-                                impl_t     impl_type,
-                                varnum_t   var_num,
-                                blksz_t*   b )
-{
-	cntl->impl_type = impl_type;
-	cntl->var_num   = var_num;
-	cntl->b         = b;
-}
-
