@@ -5,6 +5,7 @@
    libraries.
 
    Copyright (C) 2014, The University of Texas at Austin
+   Copyright (C) 2018, Advanced Micro Devices, Inc.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are
@@ -14,9 +15,9 @@
     - Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
-    - Neither the name of The University of Texas at Austin nor the names
-      of its contributors may be used to endorse or promote products
-      derived from this software without specific prior written permission.
+    - Neither the name(s) of the copyright holder(s) nor the names of its
+      contributors may be used to endorse or promote products derived
+      from this software without specific prior written permission.
 
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
@@ -48,6 +49,10 @@ void bli_obj_create
 
 	bli_obj_create_without_buffer( dt, m, n, obj );
 
+	#ifdef BLIS_ENABLE_MEM_TRACING
+	printf( "bli_obj_create(): " );
+	#endif
+
 	bli_obj_alloc_buffer( rs, cs, 1, obj );
 }
 
@@ -108,15 +113,23 @@ void bli_obj_create_without_buffer
 	bli_obj_set_elem_size( elem_size, obj );
 	bli_obj_set_target_dt( dt, obj );
 	bli_obj_set_exec_dt( dt, obj );
+	bli_obj_set_comp_dt( dt, obj );
 	bli_obj_set_dims( m, n, obj );
 	bli_obj_set_offs( 0, 0, obj );
 	bli_obj_set_diag_offset( 0, obj );
 
 	// Set the internal scalar to 1.0.
+	bli_obj_set_scalar_dt( dt, obj );
 	s = bli_obj_internal_scalar_buffer( obj );
 
-	if      ( bli_is_float( dt )    ) { bli_sset1s( *(( float*    )s) ); }
-	else if ( bli_is_double( dt )   ) { bli_dset1s( *(( double*   )s) ); }
+	// Always writing the imaginary component is needed in mixed-domain
+	// scenarios. Failing to do this can lead to reading uninitialized
+	// memory just before calling the macrokernel (as the internal scalars
+	// for A and B are merged).
+	//if      ( bli_is_float( dt )    ) { bli_sset1s( *(( float*    )s) ); }
+	//else if ( bli_is_double( dt )   ) { bli_dset1s( *(( double*   )s) ); }
+	if      ( bli_is_float( dt )    ) { bli_cset1s( *(( scomplex* )s) ); }
+	else if ( bli_is_double( dt )   ) { bli_zset1s( *(( dcomplex* )s) ); }
 	else if ( bli_is_scomplex( dt ) ) { bli_cset1s( *(( scomplex* )s) ); }
 	else if ( bli_is_dcomplex( dt ) ) { bli_zset1s( *(( dcomplex* )s) ); }
 }
@@ -224,6 +237,10 @@ void bli_obj_create_1x1
 {
 	bli_obj_create_without_buffer( dt, 1, 1, obj );
 
+	#ifdef BLIS_ENABLE_MEM_TRACING
+	printf( "bli_obj_create_1x1(): " );
+	#endif
+
 	bli_obj_alloc_buffer( 1, 1, 1, obj );
 }
 
@@ -269,7 +286,13 @@ void bli_obj_free
 		// is a detached scalar (ie: if the buffer pointer refers to the
 		// address of the internal scalar buffer).
 		if ( bli_obj_buffer( obj ) != bli_obj_internal_scalar_buffer( obj ) )
+		{
+			#ifdef BLIS_ENABLE_MEM_TRACING
+			printf( "bli_obj_free(): " );
+			#endif
+
 			bli_free_user( bli_obj_buffer( obj ) );
+		}
 	}
 }
 
@@ -578,45 +601,47 @@ void bli_obj_print
 	fprintf( file, "%s\n", label );
 	fprintf( file, "\n" );
 
-	fprintf( file, " m x n           %lu x %lu\n", ( unsigned long int )bli_obj_length( obj ),
-	                                               ( unsigned long int )bli_obj_width( obj ) );
+	fprintf( file, " m x n           %lu x %lu\n", ( unsigned long )bli_obj_length( obj ),
+	                                               ( unsigned long )bli_obj_width( obj ) );
 	fprintf( file, "\n" );
 
-	fprintf( file, " offm, offn      %lu, %lu\n", ( unsigned long int )bli_obj_row_off( obj ),
-	                                              ( unsigned long int )bli_obj_col_off( obj ) );
+	fprintf( file, " offm, offn      %lu, %lu\n", ( unsigned long )bli_obj_row_off( obj ),
+	                                              ( unsigned long )bli_obj_col_off( obj ) );
 	fprintf( file, " diagoff         %ld\n", ( signed long int )bli_obj_diag_offset( obj ) );
 	fprintf( file, "\n" );
 
 	fprintf( file, " buf             %p\n",  ( void* )bli_obj_buffer( obj ) );
-	fprintf( file, " elem size       %lu\n", ( unsigned long int )bli_obj_elem_size( obj ) );
+	fprintf( file, " elem size       %lu\n", ( unsigned long )bli_obj_elem_size( obj ) );
 	fprintf( file, " rs, cs          %ld, %ld\n", ( signed long int )bli_obj_row_stride( obj ),
 	                                              ( signed long int )bli_obj_col_stride( obj ) );
 	fprintf( file, " is              %ld\n", ( signed long int )bli_obj_imag_stride( obj ) );
-	fprintf( file, " m_padded        %lu\n", ( unsigned long int )bli_obj_padded_length( obj ) );
-	fprintf( file, " n_padded        %lu\n", ( unsigned long int )bli_obj_padded_width( obj ) );
-	fprintf( file, " pd              %lu\n", ( unsigned long int )bli_obj_panel_dim( obj ) );
-	fprintf( file, " ps              %lu\n", ( unsigned long int )bli_obj_panel_stride( obj ) );
+	fprintf( file, " m_padded        %lu\n", ( unsigned long )bli_obj_padded_length( obj ) );
+	fprintf( file, " n_padded        %lu\n", ( unsigned long )bli_obj_padded_width( obj ) );
+	fprintf( file, " pd              %lu\n", ( unsigned long )bli_obj_panel_dim( obj ) );
+	fprintf( file, " ps              %lu\n", ( unsigned long )bli_obj_panel_stride( obj ) );
 	fprintf( file, "\n" );
 
-	fprintf( file, " info            %lX\n", ( unsigned long int )(*obj).info );
-	fprintf( file, " - is complex    %lu\n", ( unsigned long int )bli_obj_is_complex( obj ) );
-	fprintf( file, " - is d. prec    %lu\n", ( unsigned long int )bli_obj_is_double_prec( obj ) );
-	fprintf( file, " - datatype      %lu\n", ( unsigned long int )bli_obj_dt( obj ) );
-	fprintf( file, " - target dt     %lu\n", ( unsigned long int )bli_obj_target_dt( obj ) );
-	fprintf( file, " - exec dt       %lu\n", ( unsigned long int )bli_obj_exec_dt( obj ) );
-	fprintf( file, " - has trans     %lu\n", ( unsigned long int )bli_obj_has_trans( obj ) );
-	fprintf( file, " - has conj      %lu\n", ( unsigned long int )bli_obj_has_conj( obj ) );
-	fprintf( file, " - unit diag?    %lu\n", ( unsigned long int )bli_obj_has_unit_diag( obj ) );
-	fprintf( file, " - struc type    %lu\n", ( unsigned long int )bli_obj_struc( obj ) >> BLIS_STRUC_SHIFT );
-	fprintf( file, " - uplo type     %lu\n", ( unsigned long int )bli_obj_uplo( obj ) >> BLIS_UPLO_SHIFT );
-	fprintf( file, "   - is upper    %lu\n", ( unsigned long int )bli_obj_is_upper( obj ) );
-	fprintf( file, "   - is lower    %lu\n", ( unsigned long int )bli_obj_is_lower( obj ) );
-	fprintf( file, "   - is dense    %lu\n", ( unsigned long int )bli_obj_is_dense( obj ) );
-	fprintf( file, " - pack schema   %lu\n", ( unsigned long int )bli_obj_pack_schema( obj ) >> BLIS_PACK_SCHEMA_SHIFT );
-	fprintf( file, " - packinv diag? %lu\n", ( unsigned long int )bli_obj_has_inverted_diag( obj ) );
-	fprintf( file, " - pack ordifup  %lu\n", ( unsigned long int )bli_obj_is_pack_rev_if_upper( obj ) );
-	fprintf( file, " - pack ordiflo  %lu\n", ( unsigned long int )bli_obj_is_pack_rev_if_lower( obj ) );
-	fprintf( file, " - packbuf type  %lu\n", ( unsigned long int )bli_obj_pack_buffer_type( obj ) >> BLIS_PACK_BUFFER_SHIFT );
+	fprintf( file, " info            %lX\n", ( unsigned long )(*obj).info );
+	fprintf( file, " - is complex    %lu\n", ( unsigned long )bli_obj_is_complex( obj ) );
+	fprintf( file, " - is d. prec    %lu\n", ( unsigned long )bli_obj_is_double_prec( obj ) );
+	fprintf( file, " - datatype      %lu\n", ( unsigned long )bli_obj_dt( obj ) );
+	fprintf( file, " - target dt     %lu\n", ( unsigned long )bli_obj_target_dt( obj ) );
+	fprintf( file, " - exec dt       %lu\n", ( unsigned long )bli_obj_exec_dt( obj ) );
+	fprintf( file, " - comp dt       %lu\n", ( unsigned long )bli_obj_comp_dt( obj ) );
+	fprintf( file, " - scalar dt     %lu\n", ( unsigned long )bli_obj_scalar_dt( obj ) );
+	fprintf( file, " - has trans     %lu\n", ( unsigned long )bli_obj_has_trans( obj ) );
+	fprintf( file, " - has conj      %lu\n", ( unsigned long )bli_obj_has_conj( obj ) );
+	fprintf( file, " - unit diag?    %lu\n", ( unsigned long )bli_obj_has_unit_diag( obj ) );
+	fprintf( file, " - struc type    %lu\n", ( unsigned long )bli_obj_struc( obj ) >> BLIS_STRUC_SHIFT );
+	fprintf( file, " - uplo type     %lu\n", ( unsigned long )bli_obj_uplo( obj ) >> BLIS_UPLO_SHIFT );
+	fprintf( file, "   - is upper    %lu\n", ( unsigned long )bli_obj_is_upper( obj ) );
+	fprintf( file, "   - is lower    %lu\n", ( unsigned long )bli_obj_is_lower( obj ) );
+	fprintf( file, "   - is dense    %lu\n", ( unsigned long )bli_obj_is_dense( obj ) );
+	fprintf( file, " - pack schema   %lu\n", ( unsigned long )bli_obj_pack_schema( obj ) >> BLIS_PACK_SCHEMA_SHIFT );
+	fprintf( file, " - packinv diag? %lu\n", ( unsigned long )bli_obj_has_inverted_diag( obj ) );
+	fprintf( file, " - pack ordifup  %lu\n", ( unsigned long )bli_obj_is_pack_rev_if_upper( obj ) );
+	fprintf( file, " - pack ordiflo  %lu\n", ( unsigned long )bli_obj_is_pack_rev_if_lower( obj ) );
+	fprintf( file, " - packbuf type  %lu\n", ( unsigned long )bli_obj_pack_buffer_type( obj ) >> BLIS_PACK_BUFFER_SHIFT );
 	fprintf( file, "\n" );
 }
 
